package cn.ibizlab.core.extensions.service;

import cn.ibizlab.core.ou.domain.IBZEmployee;
import cn.ibizlab.core.ou.service.IIBZEmployeeService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;

@Service
public class OUCoreService
{


	@Autowired
	@Lazy
	private OUModelService ouModelService;

	@Autowired
	@Lazy
	private IIBZEmployeeService iibzEmployeeService;


	public List<IBZEmployee> getEmpByOrg(String orgid)
	{
		if(StringUtils.isEmpty(orgid) || "nullorgid".equals(orgid))
			return new ArrayList<>();
		return iibzEmployeeService.list(new QueryWrapper<IBZEmployee>().eq("orgid",orgid).orderByAsc("showorder"));
	}

	public List<IBZEmployee> getEmpByDept(String deptid)
	{
		if(StringUtils.isEmpty(deptid) || "nulldeptid".equals(deptid))
			return new ArrayList<>();
		return iibzEmployeeService.list(new QueryWrapper<IBZEmployee>().eq("mdeptid",deptid).orderByAsc("showorder"));
	}

	public List<IBZEmployee> getSubEmpByOrg(String orgid)
	{
		if(StringUtils.isEmpty(orgid) || "nullorgid".equals(orgid))
			return new ArrayList<>();
		Map<String, Map<String, Set<String>>> store=ouModelService.getOrgModel();
		Map<String, Set<String>> orgmodel=this.getOrgModel(orgid);
		if(orgmodel.get("sub").size()==store.size())
			return iibzEmployeeService.list();
		return iibzEmployeeService.list(new QueryWrapper<IBZEmployee>().in("orgid",orgmodel.get("sub")));
	}

	public List<IBZEmployee> getSubEmpByDept(String deptid)
	{
		if(StringUtils.isEmpty(deptid) || "nulldeptid".equals(deptid))
			return new ArrayList<>();
		Map<String, Set<String>> deptmodel=this.getDeptModel(deptid);
		return iibzEmployeeService.list(new QueryWrapper<IBZEmployee>().in("mdeptid",deptmodel.get("sub")));
	}

	public List<IBZEmployee> getParentEmpByOrg(String orgid,boolean bRecurrence)
	{
		if(StringUtils.isEmpty(orgid) || "nullorgid".equals(orgid))
			return new ArrayList<>();
		Map<String, Set<String>> orgmodel=this.getOrgModel(orgid);
		List<String> parent=new ArrayList<>();
		for(String str:orgmodel.get("parent"))
		{
			parent.add(str);
			if(!bRecurrence)
				break;
		}
		if(parent.size()==0)
			return new ArrayList<>();
		return iibzEmployeeService.list(new QueryWrapper<IBZEmployee>().in("orgid",parent));
	}

	public List<IBZEmployee> getParentEmpByDept(String deptid,boolean bRecurrence)
	{
		if(StringUtils.isEmpty(deptid) || "nulldeptid".equals(deptid))
			return new ArrayList<>();
		Map<String, Set<String>> deptmodel=this.getDeptModel(deptid);
		List<String> parent=new ArrayList<>();
		for(String str:deptmodel.get("parent"))
		{
			parent.add(str);
			if(!bRecurrence)
				break;
		}
		if(parent.size()==0)
			return new ArrayList<>();
		return iibzEmployeeService.list(new QueryWrapper<IBZEmployee>().in("mdeptid",parent));
	}

	public Map<String, Set<String>> getOrgModel(String orgid)
	{
		Map<String, Map<String, Set<String>>> store=ouModelService.getOrgModel();
		if(store.containsKey(orgid))
		{
			Map<String, Set<String>> map = store.get(orgid);
			return map;
		}
		else
		{
			Map<String, Set<String>> map=new HashMap<>();
			map.put("parent",new LinkedHashSet<>());
			Set<String> sub=new LinkedHashSet<>();
			sub.add(orgid);
			map.put("sub",sub);
			return map;
		}
	}

	public Map<String, Set<String>> getDeptModel(String deptid)
	{
		Map<String, Map<String, Set<String>>> store=ouModelService.getDeptModel(ouModelService.getOrgModel());
		if(store.containsKey(deptid))
		{
			Map<String, Set<String>> map = store.get(deptid);
			return map;
		}
		else
		{
			Map<String, Set<String>> map=new HashMap<>();
			map.put("parent",new LinkedHashSet<>());
			Set<String> sub=new LinkedHashSet<>();
			sub.add(deptid);
			map.put("sub",sub);
			return map;
		}
	}

}
